//----------------------------------------------------------------------------
// audio_mixer - module
//----------------------------------------------------------------------------
// IMPORTANT:
// DO NOT MODIFY THIS FILE EXCEPT IN THE DESIGNATED SECTIONS.
//
// SEARCH FOR --USER TO DETERMINE WHERE CHANGES ARE ALLOWED.
//
// TYPICALLY, THE ONLY ACCEPTABLE CHANGES INVOLVE ADDING NEW
// PORTS AND GENERICS THAT GET PASSED THROUGH TO THE INSTANTIATION
// OF THE USER_LOGIC ENTITY.
//----------------------------------------------------------------------------
//
// ***************************************************************************
// ** Copyright (c) 1995-2006 Xilinx, Inc.  All rights reserved.            **
// **                                                                       **
// ** Xilinx, Inc.                                                          **
// ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
// ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
// ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
// ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
// ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
// ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
// ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
// ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
// ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
// ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
// ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
// ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
// ** FOR A PARTICULAR PURPOSE.                                             **
// **                                                                       **
// ***************************************************************************
//
//----------------------------------------------------------------------------
// Filename:          audio_mixer
// Author:	      Kevin Lui
// Version:           1.00.a
// Description:       Example FSL core (Verilog).
// Date:              Sun Feb 25 18:23:41 2007 (by Create and Import Peripheral Wizard)
// Verilog Standard:  Verilog-2001
//----------------------------------------------------------------------------
// Naming Conventions:
//   active low signals:                    "*_n"
//   clock signals:                         "clk", "clk_div#", "clk_#x"
//   reset signals:                         "rst", "rst_n"
//   generics:                              "C_*"
//   user defined types:                    "*_TYPE"
//   state machine next state:              "*_ns"
//   state machine current state:           "*_cs"
//   combinatorial signals:                 "*_com"
//   pipelined or register delay signals:   "*_d#"
//   counter signals:                       "*cnt*"
//   clock enable signals:                  "*_ce"
//   internal version of output port:       "*_i"
//   device pins:                           "*_pin"
//   ports:                                 "- Names begin with Uppercase"
//   processes:                             "*_PROCESS"
//   component instantiations:              "<ENTITY_>I_<#|FUNC>"
//----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
//
//
// Definition of Ports
// FSL_Clk             : Synchronous clock
// FSL_Rst           : System reset, should always come from FSL bus
// FSL_S_Clk       : Slave asynchronous clock
// FSL_S_Read      : Read signal, requiring next available input to be read
// FSL_S_Data      : Input data
// FSL_S_Control   : Control Bit, indicating the input data are control word
// FSL_S_Exists    : Data Exist Bit, indicating data exist in the input FSL bus
// FSL_M_Clk       : Master asynchronous clock
// FSL_M_Write     : Write signal, enabling writing to output FSL bus
// FSL_M_Data      : Output data
// FSL_M_Control   : Control Bit, indicating the output data are contol word
// FSL_M_Full      : Full Bit, indicating output FSL bus is full
//
////////////////////////////////////////////////////////////////////////////////

//----------------------------------------
// Module Section
//----------------------------------------
module audio_mixer 
	(
		// ADD USER PORTS BELOW THIS LINE 
		// -- USER ports added here 
		// ADD USER PORTS ABOVE THIS LINE 

		// DO NOT EDIT BELOW THIS LINE ////////////////////
		// Bus protocol ports, do not add or delete. 
		FSL_Clk,
		FSL_Rst,
		FSL_S_Clk,
		FSL_S_Read,
		FSL_S_Data,
		FSL_S_Control,
		FSL_S_Exists,
		FSL_M_Clk,
		FSL_M_Write,
		FSL_M_Data,
		FSL_M_Control,
		FSL_M_Full
		// DO NOT EDIT ABOVE THIS LINE ////////////////////
	);

// ADD USER PORTS BELOW THIS LINE 
// -- USER ports added here 
// ADD USER PORTS ABOVE THIS LINE 

input                                     FSL_Clk;
input                                     FSL_Rst;
output                                    FSL_S_Clk;
output                                    FSL_S_Read;
input      [0 : 31]                       FSL_S_Data;
input                                     FSL_S_Control;
input                                     FSL_S_Exists;
output                                    FSL_M_Clk;
output                                    FSL_M_Write;
output     [0 : 31]                       FSL_M_Data;
output                                    FSL_M_Control;
input                                     FSL_M_Full;

// ADD USER PARAMETERS BELOW THIS LINE 
// --USER parameters added here 
// ADD USER PARAMETERS ABOVE THIS LINE


//----------------------------------------
// Implementation Section
//----------------------------------------
// You will need to modify this example for
// MODULE audio_mixer to implement your coprocessor

   // Total number of input data.
   localparam NUMBER_OF_INPUT_WORDS  = 1;

   // Total number of output data
   localparam NUMBER_OF_OUTPUT_WORDS = 2;

   // Define the states of state machine
   localparam Init = 8'b00000001;
   localparam Idle = 8'b00000010;
   localparam Read_Ratio = 8'b00000100;
   localparam Read_Input1 = 8'b00001000;
   localparam Read_Input2 = 8'b00010000;
   localparam Mix = 8'b00100000;
   localparam Delay  = 8'b01000000;
   localparam Write_Outputs = 8'b10000000;

   reg [7:0] state;
   reg [7:0] next_state;

      
   //Hold data inputs;
   reg [0:31] input1;
   reg [0:31] input2;
   
   //Output data
   reg [31:0] data_output;
   
   //Registers to store current mixing ratio
   localparam MIX_DEFAULT = 64;
   localparam MIX_MAX = 128;
   reg [6:0] mix1;	//7 bit registers store values up to 128
   reg [6:0] mix2;	//100 is the max value either one will be

   // CAUTION:
   // The sequence in which data are read in should be
   // consistent with the sequence they are written in the
   // driver's audio_mixer.c file

   //This will ensure that the module only tries to read the FSL in the read states
   assign FSL_S_Read  = (state == Read_Ratio || state == Read_Input1 || state == Read_Input2) ? FSL_S_Exists : 0;
   
   //This will ensure that the module only tries to write to the FSL during the write state
   assign FSL_M_Write = (state == Delay || state == Write_Outputs) ? ~FSL_M_Full : 0;

   assign FSL_M_Data = data_output;
   
   //Disable sending control signals
   assign FSL_M_Control = 1'b0;


//State Machine
   always @*
   begin  // process The_SW_accelerator
      if (FSL_Rst)               // Synchronous reset (active high)
        begin
           // CAUTION: make sure your reset polarity is consistent with the
           // system reset polarity
           next_state <= Init;
        end
      else
        case (state)
	  Init:
	  begin
	    next_state <= Idle;
	  end
        
          Idle: 
            if (FSL_S_Exists == 1)
            begin
              next_state <= Read_Ratio;
            end
          
          Read_Ratio:
            if (FSL_S_Exists == 1)
            begin
              next_state <= Read_Input1;
            end

          Read_Input1: 
            if (FSL_S_Exists == 1) 
            begin
              next_state <= Read_Input2;
            end
            
          Read_Input2:
	  begin
            if (FSL_S_Exists == 1)
              next_state <= Mix;
	  end
		    
          Mix: 
	  begin	
	    next_state <= Delay;
	  end
	  
	  Delay:
	  begin
	    next_state <= Write_Outputs;
	  end
            	
          Write_Outputs:
	  begin
            next_state <= Idle;		//This will drop samples if FIFO is full
	  end
	  
	  default:
	    next_state <= Idle;
            
         endcase
   end
   
   //Perform operations depending on state
   always @(posedge FSL_Clk)
   begin
   	case (state)
   	  Init:
   	  begin
	       mix1 = MIX_DEFAULT;		//initialize ratios to defaults
	       mix2 = MIX_DEFAULT;
	     end
			
   	  Idle:
   	  begin
   	    data_output = 0;
   	    input1 = 0;
   	    input2 = 0;
   	  end
   	    
   	  Read_Ratio:
   	    if (FSL_S_Exists == 1)
   	    begin
   	      mix1 = FSL_S_Data[25:31];
   	      mix2 = MIX_MAX - mix1;
   	    end
   		
   	  Read_Input1:
   	    if (FSL_S_Exists == 1)
   	    begin
   	      input1 = FSL_S_Data;	//If not control signal, load input1 with data
   	      
   	    end
   				
   	    Read_Input2:
   	      if (FSL_S_Exists == 1)
   	      begin
   		      input2 = FSL_S_Data;
   		
   	      end

   	      
   	    Mix:
   	    begin
   	      input1 = input1*mix1/MIX_DEFAULT;
   	      input2 = input2*mix2/MIX_DEFAULT;
   	      //This is the mixing formula to prevent output volume from being far too loud
   	      data_output = input1 + input2 - input1*input2/65536;
   	    end
   	    
   	    Delay:
   	    ;
   	      
   	    Write_Outputs:
   	    ;
   			
   	endcase
   end
   
   //Assign Next State
   always @(posedge FSL_Clk)
   begin	
   	state = next_state;
   end

endmodule
